=begin
**
** Copyright (C) 2004-2005 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**

** Translated to QtRuby by Richard Dale
=end

require 'borderlayout.rb'
	
class Window < Qt::Widget
	
	def initialize(parent = nil)
		super
	    centralWidget = Qt::TextBrowser.new
	    centralWidget.plainText = tr("Central widget")
	
	    self.layout = BorderLayout.new do |l|
			l.addWidget(centralWidget, BorderLayout::Center)
			l.addWidget(createLabel("North"), BorderLayout::North)
			l.addWidget(createLabel("West"), BorderLayout::West)
			l.addWidget(createLabel("East 1"), BorderLayout::East)
			l.addWidget(createLabel("East 2") , BorderLayout::East)
			l.addWidget(createLabel("South"), BorderLayout::South)
		end
	
	    setWindowTitle(tr("Border Layout"))
	end
	
	def createLabel(text)
	    label = Qt::Label.new(text)
	    label.frameStyle = Qt::Frame::Box | Qt::Frame::Raised
	    return label
	end
end
