=begin
**
** Copyright (C) 2004-2005 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**

** Translated to QtRuby by Richard Dale
=end
    
    
    
class DomItem
    def initialize(node, row, parent = nil)
        @domNode = node
        # Record the item's location within its parent.
        @rowNumber = row
        @parentItem = parent
        @childItems = {}
    end
    
    def node()
        return @domNode
    end
    
    def parent()
        return @parentItem
    end
    
    def child(i)
        if @childItems.include?(i)
            return @childItems[i]
        end
    
        if i >= 0 && i < @domNode.childNodes().length
            childNode = @domNode.childNodes().item(i)
            childItem = DomItem.new(childNode, i, self)
            @childItems[i] = childItem
            return childItem
        end
        return nil
    end
    
    def row()
        return @rowNumber
    end
end
